''**************************************
''
''  TLC5940 Test Ver. 00.1
''
''  Timothy D. Swieter, E.I.
''  company name to go here
''
''  Updated: December 13, 2007
''
''Description:
''This program is used to test the driver for the TLC5940
''
''reference:
''
''**************************************

CON
'***************************************
'  Hardware related settings
'***************************************
  _clkmode = xtal1 + pll16x                             'Use the PLL to multiple the external clock by 16
  _xinfreq = 5_000_000                                  'An external clock of 5MHz. is used (80MHz. operation)

'***************************************
'  System Definitions      
'***************************************

  _OUTPUT       = 1             'Sets pin to output in DIRA register
  _INPUT        = 0             'Sets pin to input in DIRA register  
  _HIGH         = 1             'High=ON=1=3.3v DC
  _ON           = 1
  _LOW          = 0             'Low=OFF=0=0v DC
  _OFF          = 0
  _ENABLE       = 1             'Enable (turn on) function/mode
  _DISABLE      = 0             'Disable (turn off) function/mode

  'Used for debug string construction, remove when debug remove
  CR            = 13                                    'Go to next line?                      
  FF            = 12                                    'Clear screen
  LF            = 10                                    'Go to next line

  _DELAYTIME    = 50
  _INC          = 10
  _DEC          = 5
  _MAXCH        = 15
  _MAXINT       = 175
  
VAR               'Variables to be located here

  long  mode

OBJ               'Object declaration to be located here

  PORT0         : "TIM_TLC5940_Driver_Ver004.spin"
  PORT1         : "TIM_TLC5940_Driver_Ver004.spin"
  DMX           : "TIM_DMX_Driver_Ver013.spin"    
  Debug         : "PC_Debug.spin"                       'This object is used for debugging purposes only, remove before production

PUB MAIN | temp0

  ' Setup the Debug parameters so items can be viewed in HyperTerminal
  debug.start(460_800)                                  'start messaging
  PauseMSec(500)                                        'Wait 5 second for me to switch applications to hyperterminal
  debug.str(STRING(FF, "TLC5940 Tester", CR, LF, LF))   'print string
  PauseMSec(500)                                        'Wait 5 second for me to switch applications to hyperterminal

  'Initialize the I/O (note _gsclk and _blank defined in cog. vprg not connected for test, wired low)
  '                _sclk, _sin, _xlat, _gsclk, _blank, _vprg
  PORT0.StartTLC5940(5   , 2   , 4    , 0     , 1     , 9)
  PORT1.StartTLC5940(21  , 18  , 20   , 16    , 17    , 25)

{
 'Initialize the mode variable
  mode := 2

  'Select which mode to display
  case mode

    0:  Mode0Play               '
    1:  Mode1Play               'My test vector
    2:  Mode2Play               'Both walls cycling together slowly
    3:  Mode3Play               'DMX
    4:  Mode4Play               'DMX test
    5:  Mode5Play               'wall cycle
}
    
  temp0 := 100
  DMX.start(10, 12)  
  
  repeat
    if ina[12]
      temp0 := 100
    else
      temp0--
      if temp0 < 0
        temp0 := 0
    if temp0 == 0
      Mode2Play
'     Mode5Play
    else
      Mode4Play
       

PRI Mode0Play        

PRI Mode1Play | temp0, temp1

  repeat
    'Cycle each channel and each color individually
    repeat temp0 from 0 to _MAXCH
      'Red
      repeat temp1 from 0 to 255 step _INC
        PORT0.SetChValue(temp0, temp1, 0, 0)
        PORT0.UpdateScreen
        PORT1.SetChValue(temp0, temp1, 0, 0)
        PORT1.UpdateScreen
        PauseMSec(_DELAYTIME)
      'Green
      repeat temp1 from 0 to 255 step _INC
        PORT0.SetChValue(temp0, 0, temp1, 0)
        PORT0.UpdateScreen
        PORT1.SetChValue(temp0, 0, temp1, 0)
        PORT1.UpdateScreen
        PauseMSec(_DELAYTIME)
      'Blue
      repeat temp1 from 0 to 255 step _INC
        PORT0.SetChValue(temp0, 0, 0, temp1)
        PORT0.UpdateScreen
        PORT1.SetChValue(temp0, 0, 0, temp1)
        PORT1.UpdateScreen
        PauseMSec(_DELAYTIME)
      'Reset
      PORT0.SetChValue(temp0, 0, 0, 0)
      PORT0.UpdateScreen
      PORT1.SetChValue(temp0, 0, 0, 0)
      PORT1.UpdateScreen

    'Cycle single color on all channels
    repeat temp1 from 0 to 255 step _INC
      'Red
      repeat temp0 from 0 to _MAXCH
        PORT0.SetChValue(temp0, temp1, 0, 0)
        PORT1.SetChValue(temp0, temp1, 0, 0)
      PORT0.UpdateScreen
      PORT1.UpdateScreen
      PauseMSec(_DELAYTIME)

    'Cycle single color on all channels
    repeat temp1 from 0 to 255 step _INC
      'Green
      repeat temp0 from 0 to _MAXCH
        PORT0.SetChValue(temp0, 0, temp1, 0)
        PORT1.SetChValue(temp0, 0, temp1, 0)
      PORT0.UpdateScreen
      PORT1.UpdateScreen
      PauseMSec(_DELAYTIME)

    'Cycle single color on all channels
    repeat temp1 from 0 to 255 step _INC
      'Blue
      repeat temp0 from 0 to _MAXCH
        PORT0.SetChValue(temp0, 0, 0, temp1)
        PORT1.SetChValue(temp0, 0, 0, temp1)
      PORT0.UpdateScreen
      PORT1.UpdateScreen
      PauseMSec(_DELAYTIME)

    'Reset
    repeat temp0 from 0 to _MAXCH
      PORT0.SetChValue(temp0, 0, 0, 0)
      PORT1.SetChValue(temp0, 0, 0, 0)
    PORT0.UpdateScreen
    PORT1.UpdateScreen

    'Cycle all colors on all channels
    repeat temp1 from 0 to 255 step _INC
      repeat temp0 from 0 to _MAXCH
        PORT0.SetChValue(temp0, temp1, temp1, temp1)
        PORT1.SetChValue(temp0, temp1, temp1, temp1)        
      PORT0.UpdateScreen
      PORT1.UpdateScreen                                 
      PauseMSec(_DELAYTIME)

    'Reset
    repeat temp0 from 0 to _MAXCH
      PORT0.SetChValue(temp0, 0, 0, 0)
      PORT1.SetChValue(temp0, 0, 0, 0)      
    PORT0.UpdateScreen
    PORT1.UpdateScreen                             

    repeat 5
      repeat temp0 from 0 to _MAXCH
        PORT0.SetChValue(temp0, 0, 0, 0)
        PORT1.SetChValue(temp0, 0, 0, 0)        
      PORT0.UpdateScreen
      PORT1.UpdateScreen                                 
      
      repeat temp0 from 0 to _MAXCH
        PORT0.SetChValue(temp0, 255, 255, 255)
        PORT1.SetChValue(temp0, 255, 255, 255)        
      PORT0.UpdateScreen
      PORT1.UpdateScreen                                 

      PauseMSec(10)       

      repeat temp0 from 0 to _MAXCH
        PORT0.SetChValue(temp0, 0, 0, 0)
        PORT1.SetChValue(temp0, 0, 0, 0)        
      PORT0.UpdateScreen
      PORT1.UpdateScreen                                 
      
      repeat temp0 from 0 to _MAXCH
        PORT0.SetChValue(temp0, 255, 0, 255)
        PORT1.SetChValue(temp0, 255, 0, 255)        
      PORT0.UpdateScreen
      PORT1.UpdateScreen                                 

      PauseMSec(10)

      repeat temp0 from 0 to _MAXCH
        PORT0.SetChValue(temp0, 0, 0, 0)
        PORT1.SetChValue(temp0, 0, 0, 0)        
      PORT0.UpdateScreen
      PORT1.UpdateScreen
      
      repeat temp0 from 0 to _MAXCH
        PORT0.SetChValue(temp0, 0, 255, 255)
        PORT1.SetChValue(temp0, 0, 255, 255)        
      PORT0.UpdateScreen
      PORT1.UpdateScreen                                 

      PauseMSec(10)

      repeat temp0 from 0 to _MAXCH
        PORT0.SetChValue(temp0, 0, 0, 0)
        PORT1.SetChValue(temp0, 0, 0, 0)        
      PORT0.UpdateScreen
      PORT1.UpdateScreen                                 
      
      repeat temp0 from 0 to _MAXCH
        PORT0.SetChValue(temp0, 255, 255, 0)
        PORT1.SetChValue(temp0, 255, 255, 0)        
      PORT0.UpdateScreen
      PORT1.UpdateScreen

      PauseMSec(10)

    'Reset
    repeat temp0 from 0 to _MAXCH
      PORT0.SetChValue(temp0, 0, 0, 0)
      PORT1.SetChValue(temp0, 0, 0, 0)      
    PORT0.UpdateScreen
    PORT1.UpdateScreen                             

PRI Mode2Play | temp0, temp1

  'Reset both ports
  repeat temp0 from 0 to _MAXCH
    PORT0.SetChValue(temp0, 0, 0, 0)
    PORT1.SetChValue(temp0, 0, 0, 0)      
  PORT0.UpdateScreen
  PORT1.UpdateScreen

  'loop forever!
  repeat
  
    'Red stay, blue fade out
    repeat temp1 from _MAXINT to 0
      repeat temp0 from 0 to _MAXCH
        PORT0.SetChValue(temp0, _MAXINT, 0, temp1)
        PORT1.SetChValue(temp0, _MAXINT, 0, temp1)
      PORT0.UpdateScreen
      PORT1.UpdateScreen
      if ina[12]
        return
      PauseMSec(100)

    'wait a while
    repeat 20
      if ina[12]
        return
      PauseMSec(500)

    'Red stay, green fade in
    repeat temp1 from 0 to _MAXINT
      repeat temp0 from 0 to _MAXCH
        PORT0.SetChValue(temp0, _MAXINT, temp1, 0)
        PORT1.SetChValue(temp0, _MAXINT, temp1, 0)
      PORT0.UpdateScreen
      PORT1.UpdateScreen
      if ina[12]
        return
      PauseMSec(100)

    'wait a while
    repeat 20
      if ina[12]
        return
      PauseMSec(500)
      
    'Red fade out, green stay
    repeat temp1 from _MAXINT to 0
      repeat temp0 from 0 to _MAXCH
        PORT0.SetChValue(temp0, temp1, _MAXINT, 0)
        PORT1.SetChValue(temp0, temp1, _MAXINT, 0)
      PORT0.UpdateScreen
      PORT1.UpdateScreen
      if ina[12]
        return
      PauseMSec(100)

    'wait a while
    repeat 20
      if ina[12]
        return
      PauseMSec(500)
          
    'Green stay, blue fade in
    repeat temp1 from 0 to _MAXINT
      repeat temp0 from 0 to _MAXCH
        PORT0.SetChValue(temp0, 0, _MAXINT, temp1)
        PORT1.SetChValue(temp0, 0, _MAXINT, temp1)
      PORT0.UpdateScreen
      PORT1.UpdateScreen
      if ina[12]
        return
      PauseMSec(100)

    'wait a while
    repeat 20
      if ina[12]
        return
      PauseMSec(500)

    'Green fade out, blue stay
    repeat temp1 from _MAXINT to 0
      repeat temp0 from 0 to _MAXCH
        PORT0.SetChValue(temp0, 0, temp1, _MAXINT)
        PORT1.SetChValue(temp0, 0, temp1, _MAXINT)
      PORT0.UpdateScreen
      PORT1.UpdateScreen
      if ina[12]
        return
      PauseMSec(100)

    'wait a while
    repeat 20
      if ina[12]
        return
      PauseMSec(500)

    'Red fade in, blue stay
    repeat temp1 from 0 to _MAXINT
      repeat temp0 from 0 to _MAXCH
        PORT0.SetChValue(temp0, temp1, 0, _MAXINT)
        PORT1.SetChValue(temp0, temp1, 0, _MAXINT)
      PORT0.UpdateScreen
      PORT1.UpdateScreen
      if ina[12]
        return
      PauseMSec(100)

    'wait a while
    repeat 20
      if ina[12]
        return
      PauseMSec(500)                                                      

PRI Mode3Play | temp0, chvalues[96]


  DMX.start(10, 12)

  'Loop forever!
  repeat
    repeat temp0 from 0 to 95
      chvalues[temp0] := DMX.level(temp0+1)
    if chvalues[0] == 0
      repeat temp0 from 0 to 15
        PORT0.SetChValue(temp0, chvalues[(3*temp0)+0], chvalues[(3*temp0)+1], chvalues[(3*temp0)+2])
        PORT1.SetChValue(temp0, chvalues[(3*temp0)+48], chvalues[(3*temp0)+49], chvalues[(3*temp0)+50])
      PORT0.UpdateScreen
      PORT1.UpdateScreen

PRI Mode4Play | temp0, chvalues[96], temp1

  repeat temp0 from 0 to 95
    chvalues[temp0] := DMX.level(temp0+1)
  if chvalues[0] == 0
    repeat temp0 from 0 to 15
      PORT0.SetChValue(temp0, chvalues[(3*temp0)+0], chvalues[(3*temp0)+1], chvalues[(3*temp0)+2])
      PORT1.SetChValue(temp0, chvalues[(3*temp0)+48], chvalues[(3*temp0)+49], chvalues[(3*temp0)+50])
    PORT0.UpdateScreen
    PORT1.UpdateScreen

  temp1 := DMX.slots
  debug.dec(temp1)
  debug.str(string(CR, LF))                             'print string 

PRI Mode5Play | temp0, temp1

  'Reset both ports
  repeat temp0 from 0 to _MAXCH
    PORT0.SetChValue(temp0, 255, 255, 255)
    PORT1.SetChValue(temp0, 255, 255, 255)
  PORT0.UpdateScreen
  PORT1.UpdateScreen

{ 'loop forever!
  repeat
    'Red stay, blue fade out
    repeat temp1 from _MAXINT to 0
      repeat temp0 from 0 to _MAXCH
        PORT0.SetChValue(temp0, _MAXINT, 0, temp1)
        PORT1.SetChValue(temp0, _MAXINT, 0, temp1)
      PORT0.UpdateScreen
      PORT1.UpdateScreen
      PauseMSec(50)

    'wait a while
    PauseMSec(10000)      

    'Red stay, green fade in
    repeat temp1 from 0 to _MAXINT
      repeat temp0 from 0 to _MAXCH
        PORT0.SetChValue(temp0, _MAXINT, temp1, 0)
        PORT1.SetChValue(temp0, _MAXINT, temp1, 0)
      PORT0.UpdateScreen
      PORT1.UpdateScreen
      PauseMSec(50)

    'wait a while
    PauseMSec(10000)

    'Red fade out, green stay
    repeat temp1 from _MAXINT to 0
      repeat temp0 from 0 to _MAXCH
        PORT0.SetChValue(temp0, temp1, _MAXINT, 0)
        PORT1.SetChValue(temp0, temp1, _MAXINT, 0)
      PORT0.UpdateScreen
      PORT1.UpdateScreen
      PauseMSec(50)

    'wait a while
    PauseMSec(10000)
          
    'Green stay, blue fade in
    repeat temp1 from 0 to _MAXINT
      repeat temp0 from 0 to _MAXCH
        PORT0.SetChValue(temp0, 0, _MAXINT, temp1)
        PORT1.SetChValue(temp0, 0, _MAXINT, temp1)
      PORT0.UpdateScreen
      PORT1.UpdateScreen
      PauseMSec(50)

    'wait a while
    PauseMSec(10000)

    'Green fade out, blue stay
    repeat temp1 from _MAXINT to 0
      repeat temp0 from 0 to _MAXCH
        PORT0.SetChValue(temp0, 0, temp1, _MAXINT)
        PORT1.SetChValue(temp0, 0, temp1, _MAXINT)
      PORT0.UpdateScreen
      PORT1.UpdateScreen
      PauseMSec(50)

    'wait a while
    PauseMSec(10000)

    'Red fade in, blue stay
    repeat temp1 from 0 to _MAXINT
      repeat temp0 from 0 to _MAXCH
        PORT0.SetChValue(temp0, temp1, 0, _MAXINT)
        PORT1.SetChValue(temp0, temp1, 0, _MAXINT)
      PORT0.UpdateScreen
      PORT1.UpdateScreen
      PauseMSec(50)

    'wait a while
    PauseMSec(10000)
}
     
PRI PauseMSec(Duration)
''Pause execution in milliseconds.
''Duration = number of milliseconds to delay
  
  waitcnt(((clkfreq / 1_000 * Duration - 3932) #> 381) + cnt)